@@ -28,15 +28,12 @@ __license__ = "Apache 2.0"
 
 import numpy as np
 from kineticstoolkit.timeseries import TimeSeries, TimeSeriesEvent
-from kineticstoolkit.decorators import stable
+from kineticstoolkit.decorators import stable, experimental, directory
 import warnings
-from typing import List, Dict, Tuple
+from typing import List, Dict, Tuple, Sequence, Optional
 
 
-listing = []  # type: List[str]
-
-
-@stable(listing)
+@stable
 def detect_cycles(ts: TimeSeries,
                   data_key: str, /,
                   event_name1: str,
@@ -191,12 +188,13 @@ def detect_cycles(ts: TimeSeries,
     return tsout
 
 
-@stable(listing)
+@experimental
 def time_normalize(
         ts: TimeSeries, /,
         event_name1: str,
         event_name2: str, *,
         n_points: int = 100,
+        span: Optional[Sequence[int]] = None,
         ) -> TimeSeries:
     """
     Time-normalize cycles in a TimeSeries.
@@ -205,13 +203,8 @@ def time_normalize(
     event_name1 and event_name2 on n_points. The time-normalized cycles are
     put end to end. For example, for a TimeSeries that contains three
     cycles, a time normalization with 100 points will give a TimeSeries
-    of length 300. The TimeSeries' events are also time-normalized.
-
-    To time-normalize a cycle between two events of the same name, use '_' for
-    event_name2. For example, to time-normalize a TimeSeries between each
-    heel strike and the next heel strike:
-
-        kineticstoolkit.cycles.time_normalize(ts, 'heelstrike', '_')
+    of length 300. The TimeSeries' events are also time-normalized, including
+    event_name1 but with event_name2 renamed as '_'.
 
     Parameters
     ----------
@@ -223,46 +216,109 @@ def time_normalize(
         The event name that correspond to the end of a cycle.
     n_points
         Optional. The number of points of the output TimeSeries.
+    span
+        Optional. Specifies which normalized points to include in the output
+        TimeSeries. See note below.
 
     Returns
     -------
     TimeSeries
         A new TimeSeries where each cycle has been time-normalized.
+
+    Notes
+    -----
+    The span argument is experimental. Use it to define which normalized
+    points to include in the output TimeSeries. For example, to normalize in
+    percents and to include only data from 10 to 90% of each cycle, assign
+    100 to n_points and [10, 90] to span. The resulting TimeSeries will then
+    be expressed in percents and wrap each 80 points. It is also possible to
+    include pre-cycle or post-cycle data. For example, to normalize in
+    percents and to include 20% pre-cycle and 15% post-cycle, assign 100 to
+    n_points and [-20, 15] to span. The resulting TimeSeries will then wrap
+    each 135 points with the cycles starting at 20, 155, etc. and ending at
+    119, 254, etc. For each cycle, events outside the 0-100% spans are ignored.
+
     """
+    # Optional span
+    if span is None:
+        span = [0, n_points]
+
     # Find the final number of cycles
     if len(ts.events) < 2:
         raise(ValueError('No cycle can be defined from these event names.'))
 
-    if event_name1 == event_name2:
-        warnings.warn("It is better practice to use '_' as the second event "
-                      "name instead of repeating the same name twice.")
-        event_name2 = '_'
-
     i_cycle = 0
 
     # Initialize the destination TimeSeries
     dest_ts = ts.copy()
     dest_ts.events = []
-    dest_ts.time_info['Unit'] = '%'
+    if n_points == 100:
+        dest_ts.time_info['Unit'] = '%'
+    else:
+        dest_ts.time_info['Unit'] = f"1/{n_points}"
 
     dest_data = {}  # type: Dict[str, List[np.ndarray]]
     dest_data_shape = {}  # type: Dict[str, Tuple[int]]
 
+    # Go through all cycles
     while True:
-        # Get the TimeSeries for this cycle
-        subts = ts.get_ts_between_events(event_name1, event_name2,
-                                         i_cycle, i_cycle,
-                                         inclusive=True)
 
-        if subts.time.shape[0] == 0:  # Empty TimeSeries, retry
-            subts = ts.get_ts_between_events(event_name1, event_name1,
-                                             i_cycle, i_cycle + 1,
-                                             inclusive=True)
+        # Get the begin time for this cycle
+        begin_time = ts.get_event_time(event_name1, i_cycle)
 
-        if subts.time.shape[0] == 0:  # We are done. Quit the loop.
+        # Get the end time for this cycle
+        end_cycle = 0
+        end_time = ts.get_event_time(event_name2, end_cycle)
+        while end_time <= begin_time:
+            end_cycle += 1
+            end_time = ts.get_event_time(event_name2, end_cycle)
+
+        # We are done. Quit the loop.
+        if np.isnan(begin_time) or np.isnan(end_time):
             break
 
-        subts.trim_events()
+        # Get the extended begin and end times considering relative_span
+        extended_begin_time = (begin_time +
+                               span[0] / n_points *
+                               (end_time - begin_time))
+        extended_end_time = (begin_time +
+                             span[1] / n_points *
+                             (end_time - begin_time))
+
+        # Extract this cycle
+        subts = ts.get_ts_between_times(
+            extended_begin_time, extended_end_time, inclusive=True)
+        subts_backup = subts.copy()
+
+        if subts.time.shape[0] == 0:
+            raise ValueError("")
+
+        # Resample this cycle on span + 1
+        # (and get the first points after)
+        try:
+            subts.resample(
+                np.linspace(extended_begin_time,
+                            extended_end_time,
+                            span[1] - span[0] + 1))
+        except ValueError:
+            subts = subts_backup  # In case the try messed with subts
+            subts.resample(
+                np.linspace(extended_begin_time,
+                            extended_end_time,
+                            span[1] - span[0] + 1),
+                fill_value='extrapolate')
+            warnings.warn(f"Cycle {i_cycle} has been extrapolated.")
+
+        # Keep only the first points (the last one belongs to the next cycle)
+        subts = subts.get_ts_between_indexes(
+            0, span[1] - span[0] - 1, inclusive=True)
+
+        # Keep only the events in the unextended span
+        events = []
+        for event in subts.events:
+            if event.time >= begin_time and event.time < end_time:
+                events.append(event)
+        subts.events = events
         subts.sort_events()
 
         # Separate start/end events from the other
@@ -274,29 +330,26 @@ def time_normalize(
             else:
                 other_events.append(event)
 
-        original_start = subts.events[0].time
-        original_stop = subts.events[-1].time
+        # Add event_name1 at the beginning and end (duplicates will be
+        # cancelled at the end)
+        dest_ts.add_event(-span[0] +
+                          i_cycle * (span[1] - span[0]),
+                          event_name1)
+        dest_ts.add_event(-span[0] + n_points +
+                          i_cycle * (span[1] - span[0]),
+                          '_')
 
-        # Resample this TimeSeries on n_points
-        try:
-            subts.resample(
-                np.linspace(subts.time[0], subts.time[-1], n_points))
-        except ValueError:
-            subts.resample(
-                np.linspace(subts.time[0], subts.time[-1], n_points),
-                fill_value='extrapolate')
-            warnings.warn(f"Cycle {i_cycle} has been extrapolated.")
+        # Add the other events
+        def time_to_normalized_time(time):
+            """Resample the events times."""
+            return ((time - extended_begin_time) /
+                    (extended_end_time - extended_begin_time) *
+                    (span[1] - span[0]) + i_cycle * (span[1] - span[0]))
 
-        # Resample the events and add them to the
-        # destination TimeSeries
-        dest_ts.add_event(i_cycle * n_points, event_name1)
-        dest_ts.add_event((i_cycle + 1) * n_points - 1, '_')
         for i_event, event in enumerate(other_events):
 
             # Resample
-            new_time = ((event.time - original_start) /
-                        (original_stop - original_start) *
-                        (n_points - 1)) + i_cycle * n_points
+            new_time = time_to_normalized_time(event.time)
             dest_ts.add_event(new_time, event.name)
 
         # Add this cycle to dest_time and dest_data
@@ -311,20 +364,20 @@ def time_normalize(
     n_cycles = i_cycle
 
     # Put back dest_time and dest_data in dest_ts
-    dest_ts.time = np.arange(n_cycles * n_points)
+    dest_ts.time = 1.0 * np.arange(n_cycles * (span[1] - span[0]))
     for key in ts.data:
         # Stack the data into a [cycle, percent, values] shape
         temp = np.array(dest_data[key])
         # Reshape to put all cycles end to end
         new_shape = list(dest_data_shape[key])
-        new_shape[0] = n_cycles * n_points
+        new_shape[0] = n_cycles * (span[1] - span[0])
         dest_ts.data[key] = np.reshape(temp, new_shape)
 
     dest_ts.sort_events()
     return dest_ts
 
 
-@stable(listing)
+@stable
 def stack(ts: TimeSeries, /, n_points: int = 100) -> Dict[str, np.ndarray]:
     """
     Stack time-normalized TimeSeries' data into a dict of arrays.
@@ -365,7 +418,7 @@ def stack(ts: TimeSeries, /, n_points: int = 100) -> Dict[str, np.ndarray]:
     return data
 
 
-@stable(listing)
+@stable
 def unstack(data: Dict[str, np.ndarray], /) -> TimeSeries:
     """
     Unstack time-normalized data from a dict of arrays to a TimeSeries.
@@ -468,7 +521,7 @@ def unstack(data: Dict[str, np.ndarray], /) -> TimeSeries:
 #     return out
 
 
-@stable(listing)
+@stable
 def most_repeatable_cycles(data: np.ndarray, /) -> List[int]:
     """
     Get the indexes of the most repeatable cycles in TimeSeries or array.
@@ -548,14 +601,19 @@ def most_repeatable_cycles(data: np.ndarray, /) -> List[int]:
     set_all = set(range(n_cycles))
     set_done = set(done_cycles)
     remain = sorted(list(set_all - set_done))
-    out_cycles.append(remain[1])
-    out_cycles.append(remain[0])
+    if len(remain) > 1:
+        out_cycles.append(remain[1])
+    if len(remain) > 0:
+        out_cycles.append(remain[0])
 
     return out_cycles[-1::-1]
 
 
-def __dir__() -> List[str]:
-    return listing
+module_locals = locals()
+
+
+def __dir__():
+    return directory(module_locals)
 
 
 if __name__ == "__main__":
