@@ -25,25 +25,23 @@ __email__ = "chenier.felix@uqam.ca"
 __license__ = "Apache 2.0"
 
 
+import kineticstoolkit.geometry as geometry
 from kineticstoolkit import TimeSeries
-from kineticstoolkit.decorators import stable, unstable
-from typing import Sequence, Dict, Any, List
+from kineticstoolkit.decorators import stable, unstable, experimental, directory
+from typing import Sequence, Dict, Any
 
 import numpy as np
 import warnings
 import struct  # To unpack data from N3D files
 
 try:
-    from ezc3d import c3d as ezc3d
+    import ezc3d
 except ModuleNotFoundError:
     warnings.warn('Could not load ezc3d. Function kinematics.read_c3d_file '
                   'will not work.')
 
 
-listing = []  # type: List[str]
-
-
-@stable(listing)
+@stable
 def read_c3d_file(filename: str) -> TimeSeries:
     """
     Read markers from a C3D file.
@@ -76,7 +74,7 @@ def read_c3d_file(filename: str) -> TimeSeries:
 
     """
     # Create the reader
-    reader = ezc3d(filename)
+    reader = ezc3d.c3d(filename)
 
     # Create the output timeseries
     output = TimeSeries()
@@ -114,7 +112,60 @@ def read_c3d_file(filename: str) -> TimeSeries:
     return output
 
 
-@stable(listing)
+@unstable
+def write_c3d_file(filename: str, markers: TimeSeries) -> None:
+    """
+    Write a markers TimeSeries to a C3D file.
+
+    This is the mirror operation to read_c3d_file.
+
+    Parameters
+    ----------
+    filename
+        Path of the C3D file
+
+    markers
+        Markers trajectories, following the same form as the TimeSeries
+        read by read_c3d_file.
+
+    Notes
+    -----
+    - This function relies on `ezc3d`, which is available on
+      conda-forge and on git-hub. Please install ezc3d before using
+      read_c3d_file. https://github.com/pyomeca/ezc3d
+
+    - The point unit in the output c3d is m.
+
+    """
+    sample_rate = ((markers.time.shape[0] - 1) /
+                   (markers.time[-1] - markers.time[0]))
+
+    marker_list = []
+    marker_data = np.zeros((4, len(markers.data), len(markers.time)))
+
+    for i_marker, marker in enumerate(markers.data):
+        marker_list.append(marker)
+        marker_data[0, i_marker, :] = markers.data[marker][:, 0]
+        marker_data[1, i_marker, :] = markers.data[marker][:, 1]
+        marker_data[2, i_marker, :] = markers.data[marker][:, 2]
+        marker_data[3, i_marker, :] = markers.data[marker][:, 3]
+
+    # Load an empty c3d structure
+    c3d = ezc3d.c3d()
+
+    # Fill it with data
+    c3d['parameters']['POINT']['RATE']['value'] = [sample_rate]
+    c3d['parameters']['POINT']['LABELS']['value'] = tuple(marker_list)
+    c3d['data']['points'] = marker_data
+
+    # Add a custom parameter to the POINT group
+    c3d.add_parameter('POINT', 'UNITS', 'm')
+
+    # Write the data
+    c3d.write(filename)
+
+
+@stable
 def read_n3d_file(filename: str, labels: Sequence[str] = []):
     """
     Read markers from an NDI N3D file.
@@ -302,7 +353,7 @@ def read_n3d_file(filename: str, labels: Sequence[str] = []):
 #            return(the_timeseries)
 
 
-@unstable(listing)
+@experimental
 def create_rigid_body_config(
     markers: TimeSeries,
     marker_names: Sequence[str]
@@ -340,7 +391,10 @@ def create_rigid_body_config(
     # Remove samples where at least one marker is invisible
     global_points = global_points[~geometry.isnan(global_points)]
 
-    rigid_bodies = geometry.create_reference_frames(global_points)
+    rigid_bodies = geometry.create_frames(
+        origin=global_points[:, :, 0],
+        x=global_points[:, :, 1] - global_points[:, :, 0],
+        xy=global_points[:, :, 2] - global_points[:, :, 0])
     local_points = geometry.get_local_coordinates(
             global_points, rigid_bodies)
 
@@ -354,7 +408,7 @@ def create_rigid_body_config(
             }
 
 
-@unstable(listing)
+@experimental
 def register_markers(
         markers: TimeSeries,
         rigid_body_configs: Dict[str, Dict[str, Any]],
@@ -414,7 +468,7 @@ def register_markers(
     return rigid_bodies
 
 
-@unstable(listing)
+@experimental
 def create_virtual_marker_config(
         markers: TimeSeries,
         rigid_bodies: TimeSeries,
@@ -465,7 +519,7 @@ def create_virtual_marker_config(
             'LocalPoint': local_points}
 
 
-@unstable(listing)
+@unstable
 def write_trc_file(markers: TimeSeries, filename: str) -> None:
     """
     Export a markers TimeSeries to OpenSim's TRC file format.
@@ -521,5 +575,8 @@ def write_trc_file(markers: TimeSeries, filename: str) -> None:
             fid.write('\n')
 
 
+module_locals = locals()
+
+
 def __dir__():
-    return listing
+    return directory(module_locals)
