@@ -30,21 +30,22 @@ __license__ = "Apache 2.0"
 
 
 import kineticstoolkit._repr
-from kineticstoolkit.decorators import stable, experimental, unstable
+from kineticstoolkit.decorators import stable, experimental, unstable, directory
 import matplotlib as mpl
 import matplotlib.pyplot as plt
 import numpy as np
 import scipy as sp
 import pandas as pd
+
 import warnings
 from ast import literal_eval
 from copy import deepcopy
-from typing import Dict, List, Tuple, Any, Union
+from typing import Dict, List, Tuple, Any, Union, Optional
 
 import kineticstoolkit as ktk  # For doctests
 
 
-ts_listing = []  # type: List[str]
+window_placement = {'top': 50, 'right': 0}
 
 
 def dataframe_to_dict_of_arrays(
@@ -392,6 +393,9 @@ class TimeSeries():
         self.data_info = data_info.copy()
         self.events = events.copy()
 
+    def __dir__(self):
+        return directory(TimeSeries.__dict__)
+
     def __str__(self):
         """
         Print a textual descriptive of the TimeSeries contents.
@@ -447,7 +451,7 @@ class TimeSeries():
 
         return True
 
-    @stable(ts_listing)
+    @stable
     def to_dataframe(self) -> pd.DataFrame:
         """
         Create a DataFrame by reshaping all data to one bidimensional table.
@@ -466,7 +470,7 @@ class TimeSeries():
         df.index = self.time
         return df
 
-    @stable(ts_listing)
+    @stable
     def from_dataframe(dataframe: pd.DataFrame, /) -> 'TimeSeries':
         """
         Create a new TimeSeries from a Pandas Dataframe
@@ -494,7 +498,7 @@ class TimeSeries():
         ts.time = dataframe.index.to_numpy()
         return ts
 
-    @stable(ts_listing)
+    @stable
     def add_data_info(self, data_key: str, info_key: str, value: Any) -> None:
         """
         Add metadata to TimeSeries' data.
@@ -529,12 +533,12 @@ class TimeSeries():
         {'Color': [43, 2, 255]}
 
         """
-        if data_key in self.data_info:   # Assign the value
+        try:
             self.data_info[data_key][info_key] = value
-        else:  # Create and assign value
+        except KeyError:
             self.data_info[data_key] = {info_key: value}
 
-    @stable(ts_listing)
+    @stable
     def remove_data_info(self, data_key: str, info_key: str, /) -> None:
         """
         Remove metadata from a TimeSeries' data.
@@ -567,7 +571,7 @@ class TimeSeries():
         except KeyError:
             pass
 
-    @stable(ts_listing)
+    @stable
     def rename_data(self, old_data_key: str, new_data_key: str, /) -> None:
         """
         Rename a key in data and data_info.
@@ -600,12 +604,17 @@ class TimeSeries():
         {'signal': {'Unit': 'm'}}
 
         """
-        if old_data_key in self.data:
+        try:
             self.data[new_data_key] = self.data.pop(old_data_key)
-        if old_data_key in self.data_info:
-            self.data_info[new_data_key] = self.data_info.pop(old_data_key)
+        except KeyError:
+            pass
 
-    @stable(ts_listing)
+        try:
+            self.data_info[new_data_key] = self.data_info.pop(old_data_key)
+        except KeyError:
+            pass
+
+    @stable
     def remove_data(self, data_key: str, /) -> None:
         """
         Remove a data key and its associated metadata.
@@ -651,7 +660,7 @@ class TimeSeries():
         except KeyError:
             pass
 
-    @stable(ts_listing)
+    @stable
     def add_event(self, time: float, name: str = 'event') -> None:
         """
         Add an event to the TimeSeries.
@@ -676,11 +685,13 @@ class TimeSeries():
         """
         self.events.append(TimeSeriesEvent(time, name))
 
-    @stable(ts_listing)
-    def remove_event(self,
-                     event_name: str, event_occurrence : int = 0, /) -> None:
+    @stable
+    def rename_event(self,
+                     event_name: str,
+                     new_name: str,
+                     event_occurrence: Optional[int] = None, /) -> None:
         """
-        Remove an event.
+        Rename an event occurrence or all events of a same name.
 
         Parameters
         ----------
@@ -689,6 +700,59 @@ class TimeSeries():
         event_occurrence
             Optional. i_th occurence of the event to look for in the events
             list, starting at 0, where the occurrences are sorted in time.
+            If None (default), all occurences of this event name or renamed.
+
+        Example
+        -------
+        >>> ts = ktk.TimeSeries()
+        >>> ts.add_event(5.5, 'event1')
+        >>> ts.add_event(10.8, 'event2')
+        >>> ts.add_event(2.3, 'event2')
+
+        >>> ts.events
+        [[5.5, 'event1'], [10.8, 'event2'], [2.3, 'event2']]
+
+        >>> ts.rename_event('event2', 'event3')
+        >>> ts.events
+        [[5.5, 'event1'], [10.8, 'event3'], [2.3, 'event3']]
+
+        >>> ts.rename_event('event3', 'event4', 0)
+        >>> ts.events
+        [[5.5, 'event1'], [10.8, 'event3'], [2.3, 'event4']]
+
+        """
+        if event_name == new_name:
+            return  # Nothing to do.
+
+        if event_occurrence is None:
+            index = self.get_event_index(event_name, 0)
+            while ~np.isnan(index):
+                self.events[index].name = new_name
+                index = self.get_event_index(event_name, 0)
+
+        else:
+            index = self.get_event_index(event_name, event_occurrence)
+            try:
+                self.events[index].name = new_name
+            except TypeError:  # index was NaN
+                warnings.warn(f"The occurrence {event_occurrence} of event "
+                              f"{event_name} could not be found.")
+
+    @stable
+    def remove_event(self,
+                     event_name: str,
+                     event_occurrence: Optional[int] = None, /) -> None:
+        """
+        Remove an event occurrence or all events of a same name.
+
+        Parameters
+        ----------
+        event_name
+            Name of the event to look for in the events list.
+        event_occurrence
+            Optional. i_th occurence of the event to look for in the events
+            list, starting at 0, where the occurrences are sorted in time.
+            If None (default), all occurences of this event name or removed.
 
         Example
         -------
@@ -710,16 +774,21 @@ class TimeSeries():
         [[2.3, 'event2']]
 
         """
-        time_tolerance = 1E-15
-        event_time = self.get_event_time(event_name, event_occurrence)
-        new_events = []
-        for event in self.events:
-            if (np.abs(event.time - event_time) > time_tolerance or
-                    event.name != event_name):
-                new_events.append(event)
-        self.events = new_events
+        if event_occurrence is None:  # Remove all occurrences
+            event_index = self.get_event_index(event_name, 0)
+            while ~np.isnan(event_index):
+                self.events.pop(event_index)
+                event_index = self.get_event_index(event_name, 0)
 
-    @stable(ts_listing)
+        else:  # Remove only the specified occurrence
+            event_index = self.get_event_index(event_name, event_occurrence)
+            try:
+                self.events.pop(event_index)
+            except TypeError:
+                warnings.warn(f"The occurrence {event_occurrence} of event "
+                              f"{event_name} could not be found.")
+
+    @stable
     def ui_add_event(self,
                      name: str = 'event',
                      plot: Union[str, List[str]] = [], /, *,
@@ -758,7 +827,7 @@ class TimeSeries():
                 f'Adding the event "{name}".\n'
                 'Please zoom on the location to \n'
                 'add the event, then click Next.',
-                ['Cancel', 'Next'])
+                ['Cancel', 'Next'], **window_placement)
 
             if button <= 0:  # Cancel
                 plt.close(fig)
@@ -768,14 +837,15 @@ class TimeSeries():
                 kineticstoolkit.gui.message(
                     'Left-click to add events; \n'
                     'Right-click to delete; \n'
-                    'ENTER to finish.')
+                    'ENTER to finish.', **window_placement)
                 plt.pause(0.001)  # Update the plot
                 coordinates = plt.ginput(99999)
                 kineticstoolkit.gui.message('')
 
             else:
                 kineticstoolkit.gui.message(
-                    'Please left-click on the event to add.')
+                    'Please left-click on the event to add.',
+                    **window_placement)
                 coordinates = plt.ginput(1)
                 kineticstoolkit.gui.message('')
 
@@ -789,7 +859,7 @@ class TimeSeries():
                 button = kineticstoolkit.gui.button_dialog(
                     f'Adding the event "{name}".\n'
                     'Do you want to add more of these events?',
-                    ['Cancel', 'Add more', "Finished"])
+                    ['Cancel', 'Add more', "Finished"], **window_placement)
                 if button <= 0:  # Cancel
                     plt.close(fig)
                     return False
@@ -803,7 +873,7 @@ class TimeSeries():
         self.events = ts.events  # Add the events to self.
         return True
 
-    @stable(ts_listing)
+    @stable
     def sort_events(self, *, unique: bool = True) -> None:
         """
         Sorts the TimeSeries' events from the earliest to the latest.
@@ -842,15 +912,15 @@ class TimeSeries():
                         (self.events[i].name == self.events[i - 1].name)):
                     self.events.pop(i)
 
-    @stable(ts_listing)
+    @stable
     def copy(self) -> 'TimeSeries':
         """Deep copy of a TimeSeries."""
         return deepcopy(self)
 
-    @stable(ts_listing)
+    @stable
     def plot(self,
              data_keys: Union[str, List[str]] = [],
-             /, *,
+             /, *args,
              event_names: bool = True,
              legend: bool = True,
              **kwargs) -> None:
@@ -866,7 +936,8 @@ class TimeSeries():
         legend
             Optional. True to plot a legend, False otherwise.
 
-        Additional keyboard arguments are passed to the pyplot's plot function.
+        Additional positional and keyboard arguments are passed to the
+        pyplot's plot function.
 
         Example
         -------
@@ -884,6 +955,9 @@ class TimeSeries():
         else:
             ts = self.get_subset(data_keys)
 
+        if len(ts.data) == 0:
+            raise ValueError('There is no data to plot.')
+
         # Sort events to help finding each event's occurrence
         ts.sort_events(unique=False)
 
@@ -899,7 +973,7 @@ class TimeSeries():
         # Plot the curves
         for i_label, label in enumerate(labels):
             axes.plot(df.index.to_numpy(),
-                     df[label].to_numpy(), label=label, **kwargs)
+                      df[label].to_numpy(), *args, label=label, **kwargs)
 
         # Add labels
         plt.xlabel('Time (' + ts.time_info['Unit'] + ')')
@@ -971,7 +1045,7 @@ class TimeSeries():
             axes.legend(loc=legend_location,
                        ncol=1 + int(len(labels) / 40))  # Max 40 items per line
 
-    @stable(ts_listing)
+    @stable
     def get_index_at_time(self, time: float, /) -> int:
         """
         Get the time index that is the closest to the specified time.
@@ -1002,7 +1076,7 @@ class TimeSeries():
         """
         return np.argmin(np.abs(self.time - float(time)))
 
-    @stable(ts_listing)
+    @stable
     def get_index_before_time(self, time: float, /, *,
                               inclusive: bool = False) -> int:
         """
@@ -1031,27 +1105,41 @@ class TimeSeries():
         >>> ts.get_index_before_time(1)
         1
 
-        >>> ts.get_index_before_time(1, inclusive=True)
-        2
-
         >>> ts.get_index_before_time(1.1)
         2
 
-        >>> ts.get_index_before_time(-1)
+        >>> ts.get_index_before_time(1.1, inclusive=True)
+        3
+
+        >>> ts.get_index_before_time(0)
         nan
 
+        >>> ts.get_index_before_time(0, inclusive=True)
+        0
+
         """
+        # Edge case
+        if inclusive and time == self.time[0]:
+            return 0
+
+        # Other cases
         diff = float(time) - self.time
-        if inclusive:
-            diff[diff < 0] = np.nan
-        else:
-            diff[diff <= 0] = np.nan
+        diff[diff <= 0] = np.nan
+
         if np.all(np.isnan(diff)):  # All nans
             return np.nan
-        else:
-            return np.nanargmin(diff)
 
-    @stable(ts_listing)
+        index = np.nanargmin(diff)
+
+        if inclusive and self.time[index] < time:
+            index += 1
+
+        if index < self.time.shape[0]:
+            return index
+        else:
+            return np.nan
+
+    @stable
     def get_index_after_time(self, time: float, /, *,
                              inclusive: bool = False) -> int:
         """
@@ -1077,30 +1165,90 @@ class TimeSeries():
         >>> ts.get_index_after_time(0.9)
         2
 
+        >>> ts.get_index_after_time(0.9, inclusive=True)
+        1
+
         >>> ts.get_index_after_time(1)
         3
 
         >>> ts.get_index_after_time(1, inclusive=True)
         2
 
-        >>> ts.get_index_after_time(1.1)
-        3
-
-        >>> ts.get_index_after_time(13)
+        >>> ts.get_index_after_time(2)
         nan
 
+        >>> ts.get_index_after_time(2, inclusive=True)
+        4
+
         """
+        # Edge case
+        if inclusive and time == self.time[-1]:
+            return self.time.shape[0] - 1
+
+        # Other cases
         diff = self.time - float(time)
-        if inclusive:
-            diff[diff < 0] = np.nan
-        else:
-            diff[diff <= 0] = np.nan
+        diff[diff <= 0] = np.nan
+
         if np.all(np.isnan(diff)):  # All nans
             return np.nan
-        else:
-            return np.nanargmin(diff)
 
-    @stable(ts_listing)
+        index = np.nanargmin(diff)
+
+        if inclusive and self.time[index] > time:
+            index -= 1
+
+        if index >= 0:
+            return index
+        else:
+            return np.nan
+
+    @stable
+    def get_event_index(self,
+                        event_name: str,
+                        event_occurrence: int) -> int:
+        """
+        Get the index of a given occurrence of an event name.
+
+        Parameters
+        ----------
+        event_name
+            Name of the event to look for in the events list.
+        event_occurrence
+            i_th occurence of the event to look for in the events
+            list, starting at 0, where the occurrences are sorted in time.
+
+        Returns
+        -------
+        int
+            The index of the event.
+
+        """
+        event_occurrence = int(event_occurrence)
+
+        if event_occurrence < 0:
+            raise ValueError('event_occurrence must be positive')
+
+        # Make a list of event times, with NaNs for events whose name doesn't
+        # match what we're looking for.
+        event_times = (
+            [event.time if event.name == event_name else np.nan
+             for event in self.events])
+
+        # Sort in time
+        event_indexes = np.argsort(event_times)
+
+        # Remove indexes that correspond to nan time (wrong events)
+        clean_event_indexes = (
+            [index if ~np.isnan(event_times[index]) else np.nan
+             for index in event_indexes])
+
+        # Get the event occurrence
+        try:
+            return clean_event_indexes[event_occurrence]
+        except IndexError:
+            return np.nan
+
+    @stable
     def get_event_time(self, event_name: str,
                        event_occurrence: int = 0, /) -> float:
         """
@@ -1138,25 +1286,13 @@ class TimeSeries():
         10.8
 
         """
-        event_occurrence = int(event_occurrence)
-
-        if event_occurrence < 0:
-            raise ValueError('event_occurrence must be positive')
-
-        the_event_times = np.array([x.time for x in self.events])
-        the_event_indices = [(x.name == event_name) for x in self.events]
-
-        # Keep only the events with the specified name
-        the_event_times = np.array(the_event_times[the_event_indices])
-
-        n_events = len(the_event_times)
-        if n_events == 0 or event_occurrence >= n_events:
+        event_index = self.get_event_index(event_name, event_occurrence)
+        try:
+            return self.events[event_index].time
+        except TypeError:
             return np.nan
-        else:
-            the_event_times = np.sort(the_event_times)
-            return the_event_times[event_occurrence]
 
-    @stable(ts_listing)
+    @stable
     def get_ts_at_time(self, time: float, /) -> 'TimeSeries':
         """
         Get a one-data subset of the TimeSeries at the nearest time.
@@ -1195,7 +1331,7 @@ class TimeSeries():
             out_ts.data[the_data] = out_ts.data[the_data][index]
         return out_ts
 
-    @stable(ts_listing)
+    @stable
     def get_ts_at_event(self, event_name: str,
                         event_occurrence: int = 0, /) -> 'TimeSeries':
         """
@@ -1218,7 +1354,7 @@ class TimeSeries():
         time = self.get_event_time(event_name, event_occurrence)
         return self.get_ts_at_time(time)
 
-    @stable(ts_listing)
+    @stable
     def get_ts_before_index(self, index: int, /, *,
                             inclusive: bool = False) -> 'TimeSeries':
         """
@@ -1258,7 +1394,7 @@ class TimeSeries():
             out_ts.data[the_data] = out_ts.data[the_data][index_range]
         return out_ts
 
-    @stable(ts_listing)
+    @stable
     def get_ts_after_index(self, index: int, /, *,
                            inclusive: bool = False) -> 'TimeSeries':
         """
@@ -1298,7 +1434,7 @@ class TimeSeries():
             out_ts.data[the_data] = out_ts.data[the_data][index_range]
         return out_ts
 
-    @stable(ts_listing)
+    @stable
     def get_ts_between_indexes(self, index1: int, index2: int, /, *,
                                inclusive: bool = False) -> 'TimeSeries':
         """
@@ -1338,7 +1474,7 @@ class TimeSeries():
             out_ts.data[the_data] = out_ts.data[the_data][index_range]
         return out_ts
 
-    @stable(ts_listing)
+    @stable
     def get_ts_before_time(self, time: float, /, *,
                            inclusive: bool = False) -> 'TimeSeries':
         """
@@ -1364,19 +1500,15 @@ class TimeSeries():
         array([0. , 0.1, 0.2, 0.3])
 
         """
-        out_ts = self.copy()
+        #Edge case
+        if time > self.time[-1]:
+            return self.copy()
+
+        #Other cases
         index = self.get_index_before_time(time, inclusive=inclusive)
-        if np.isnan(index):
-            index_range = range(0)
-        else:
-            index_range = range(0, index + 1)
+        return self.get_ts_before_index(index, inclusive=True)
 
-        out_ts.time = out_ts.time[index_range]
-        for the_data in out_ts.data.keys():
-            out_ts.data[the_data] = out_ts.data[the_data][index_range]
-        return out_ts
-
-    @stable(ts_listing)
+    @stable
     def get_ts_after_time(self, time: float, /, *,
                           inclusive: bool = False) -> 'TimeSeries':
         """
@@ -1401,15 +1533,18 @@ class TimeSeries():
         >>> ts.get_ts_after_time(0.3, inclusive=True).time
         array([0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9])
 
+        >>> ts.get_ts_after_time(0.25, inclusive=True).time
+        array([0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9])
         """
-        if inclusive:
-            index = self.get_index_before_time(time, inclusive=True)
-        else:
-            index = self.get_index_after_time(time, inclusive=False)
+        #Edge case
+        if time < self.time[0]:
+            return self.copy()
 
+        #Other cases
+        index = self.get_index_after_time(time, inclusive=inclusive)
         return self.get_ts_after_index(index, inclusive=True)
 
-    @stable(ts_listing)
+    @stable
     def get_ts_between_times(self, time1: float, time2: float, /, *,
                              inclusive: bool = False) -> 'TimeSeries':
         """
@@ -1442,7 +1577,7 @@ class TimeSeries():
                                            inclusive=inclusive)
         return new_ts
 
-    @stable(ts_listing)
+    @stable
     def get_ts_before_event(self, event_name: str,
                             event_occurrence: int = 0, /, *,
                             inclusive: bool = False) -> 'TimeSeries':
@@ -1481,14 +1616,9 @@ class TimeSeries():
 
         """
         time = self.get_event_time(event_name, event_occurrence)
-        if inclusive:
-            index = self.get_index_after_time(time, inclusive=True)
-        else:
-            index = self.get_index_before_time(time, inclusive=False)
+        return self.get_ts_before_time(time, inclusive=inclusive)
 
-        return self.get_ts_before_index(index, inclusive=True)
-
-    @stable(ts_listing)
+    @stable
     def get_ts_after_event(self, event_name: str,
                            event_occurrence: int = 0, /, *,
                            inclusive: bool = False) -> 'TimeSeries':
@@ -1527,14 +1657,9 @@ class TimeSeries():
 
         """
         time = self.get_event_time(event_name, event_occurrence)
-        if inclusive:
-            index = self.get_index_before_time(time, inclusive=True)
-        else:
-            index = self.get_index_after_time(time, inclusive=False)
+        return self.get_ts_after_time(time, inclusive=inclusive)
 
-        return self.get_ts_after_index(index, inclusive=True)
-
-    @stable(ts_listing)
+    @stable
     def get_ts_between_events(self, event_name1: str, event_name2: str,
                               event_occurrence1: int = 0,
                               event_occurrence2: int = 0,
@@ -1574,7 +1699,7 @@ class TimeSeries():
                                     inclusive=inclusive)
         return ts
 
-    @stable(ts_listing)
+    @stable
     def ui_get_ts_between_clicks(
             self,
             data_keys: Union[str, List[str]] = [], /, *,
@@ -1593,7 +1718,8 @@ class TimeSeries():
         """
         fig = plt.figure()
         self.plot(data_keys)
-        kineticstoolkit.gui.message('Click on both sides of the portion to keep.')
+        kineticstoolkit.gui.message(
+            'Click on both sides of the portion to keep.', **window_placement)
         plt.pause(0.001)  # Redraw
         points = plt.ginput(2)
         kineticstoolkit.gui.message('')
@@ -1602,7 +1728,7 @@ class TimeSeries():
         return self.get_ts_between_times(min(times), max(times),
                                          inclusive=inclusive)
 
-    @stable(ts_listing)
+    @stable
     def isnan(self, data_key: str, /) -> np.ndarray:
         """
         Return a boolean array of missing samples.
@@ -1625,7 +1751,7 @@ class TimeSeries():
             values = np.sum(values, 1)
         return np.isnan(values)
 
-    @experimental(ts_listing)
+    @experimental
     def fill_missing_samples(self, max_missing_samples: int, *,
                              method: str = 'linear') -> None:
         """
@@ -1673,7 +1799,7 @@ class TimeSeries():
 
             self.data[data] = ts.data[data]
 
-    @stable(ts_listing)
+    @stable
     def shift(self, time: float, /) -> None:
         """
         Shift time and events.time.
@@ -1688,7 +1814,7 @@ class TimeSeries():
             event.time = event.time + time
         self.time = self.time + time
 
-    @stable(ts_listing)
+    @stable
     def sync_event(self, event_name: str,
                    event_occurrence: int = 0, /) -> None:
         """
@@ -1704,7 +1830,7 @@ class TimeSeries():
         """
         self.shift(-self.get_event_time(event_name, event_occurrence))
 
-    @stable(ts_listing)
+    @stable
     def trim_events(self) -> None:
         """
         Delete the events that are outside the TimeSeries' time vector.
@@ -1728,13 +1854,17 @@ class TimeSeries():
         [[0.0, 'event'], [5.0, 'event'], [9.0, 'event']]
 
         """
+        if self.time.shape[0] == 0:  # no time, thus no event to keep.
+            self.events = []
+            return
+
         events = self.events
         self.events = []
         for event in events:
             if event.time >= self.time[0] and event.time <= self.time[-1]:
                 self.add_event(event.time, event.name)
 
-    @experimental(ts_listing)
+    @experimental
     def ui_sync(self,
                 data_keys: Union[str, List[str]] = [],
                 ts2: Union['TimeSeries', None] = None,
@@ -1773,12 +1903,13 @@ class TimeSeries():
             self.plot(data_keys)
             choice = kineticstoolkit.gui.button_dialog(
                 'Please zoom on the time zero and press Next.',
-                ['Cancel', 'Next'])
+                ['Cancel', 'Next'], **window_placement)
             if choice != 1:
                 plt.close(fig)
                 return
 
-            kineticstoolkit.gui.message('Click on the sync event.')
+            kineticstoolkit.gui.message('Click on the sync event.',
+                                        **window_placement)
             click = plt.ginput(1)
             kineticstoolkit.gui.message('')
             plt.close(fig)
@@ -1816,11 +1947,11 @@ class TimeSeries():
                              'Zero both TimeSeries, using ts1',
                              'Zero both TimeSeries, using ts2',
                              'Sync both TimeSeries on a common event',
-                             'Finished'])
+                             'Finished'], **window_placement)
 
                 if choice == 0:  # Zero ts1 only
                     kineticstoolkit.gui.message(
-                        'Click on the time zero in ts1.')
+                        'Click on the time zero in ts1.', **window_placement)
                     click_1 = plt.ginput(1)
                     kineticstoolkit.gui.message('')
 
@@ -1828,7 +1959,7 @@ class TimeSeries():
 
                 elif choice == 1:  # Zero ts2 only
                     kineticstoolkit.gui.message(
-                        'Click on the time zero in ts2.')
+                        'Click on the time zero in ts2.', **window_placement)
                     click_1 = plt.ginput(1)
                     kineticstoolkit.gui.message('')
 
@@ -1836,7 +1967,7 @@ class TimeSeries():
 
                 elif choice == 2:  # Zero ts1 and ts2 using ts1
                     kineticstoolkit.gui.message(
-                        'Click on the time zero in ts1.')
+                        'Click on the time zero in ts1.', **window_placement)
                     click_1 = plt.ginput(1)
                     kineticstoolkit.gui.message('')
 
@@ -1845,7 +1976,7 @@ class TimeSeries():
 
                 elif choice == 3:  # Zero ts1 and ts2 using ts2
                     kineticstoolkit.gui.message(
-                        'Click on the time zero in ts2.')
+                        'Click on the time zero in ts2.', **window_placement)
                     click_2 = plt.ginput(1)
                     kineticstoolkit.gui.message('')
 
@@ -1854,10 +1985,11 @@ class TimeSeries():
 
                 elif choice == 4:  # Sync on a common event
                     kineticstoolkit.gui.message(
-                        'Click on the sync event in ts1.')
+                        'Click on the sync event in ts1.', **window_placement)
                     click_1 = plt.ginput(1)
                     kineticstoolkit.gui.message(
-                        'Now click on the same event in ts2.')
+                        'Now click on the same event in ts2.',
+                        **window_placement)
                     click_2 = plt.ginput(1)
                     kineticstoolkit.gui.message('')
 
@@ -1868,7 +2000,7 @@ class TimeSeries():
                     plt.close(fig)
                     finished = True
 
-    @stable(ts_listing)
+    @stable
     def get_subset(self, data_keys: Union[str, List[str]]) -> 'TimeSeries':
         """
         Return a subset of the TimeSeries.
@@ -1910,14 +2042,19 @@ class TimeSeries():
         ts.events = deepcopy(self.events)
 
         for key in data_keys:
-            if key in self.data:
+            try:
                 ts.data[key] = self.data[key].copy()
-            if key in self.data_info:
+            except KeyError:
+                pass
+
+            try:
                 ts.data_info[key] = deepcopy(self.data_info[key])
+            except KeyError:
+                pass
 
         return ts
 
-    @unstable(ts_listing)
+    @unstable
     def resample(self,
                  new_time: np.ndarray,
                  kind: str = 'linear', *,
@@ -1972,7 +2109,7 @@ class TimeSeries():
 
             if sum(index) < 3:  # Only Nans, cannot interpolate.
                 warnings.warn(
-                    f'Warning: Almost only NaNs found in signal {key}.')
+                    f'Warning: Almost only NaNs found in signal "{key}.')
                 # We generate an array of nans of the expected size.
                 new_shape = [len(new_time)]
                 for i in range(1, len(self.data[key].shape)):
@@ -2018,7 +2155,7 @@ class TimeSeries():
 
         self.time = new_time
 
-    @experimental(ts_listing)
+    @experimental
     def merge(self,
               ts: 'TimeSeries',
               data_keys: Union[str, List[str]] = [], /, *,
@@ -2093,11 +2230,6 @@ class TimeSeries():
             self.events.append(event)
         self.sort_events()
 
-    def __dir__(self):
-        """Dir for the TimeSeries."""
-        print('allo')
-        return ts_listing
-
 
 if __name__ == "__main__":
     import doctest
